=== modified file 'bzrlib/plugins/weave_fmt/repository.py'
--- bzrlib/plugins/weave_fmt/repository.py	2011-05-08 11:19:58 +0000
+++ bzrlib/plugins/weave_fmt/repository.py	2011-05-08 11:34:17 +0000
@@ -50,7 +50,6 @@
 from bzrlib.decorators import needs_read_lock, needs_write_lock
 from bzrlib.repository import (
     InterRepository,
-    InterSameDataRepository,
     RepositoryFormat,
     )
 from bzrlib.store.text import TextStore
@@ -60,6 +59,7 @@
     VersionedFiles,
     )
 from bzrlib.vf_repository import (
+    InterSameDataRepository,
     VersionedFileCommitBuilder,
     VersionedFileRepository,
     VersionedFileRepositoryFormat,

=== modified file 'bzrlib/repofmt/knitrepo.py'
--- bzrlib/repofmt/knitrepo.py	2011-05-08 11:10:16 +0000
+++ bzrlib/repofmt/knitrepo.py	2011-05-08 11:34:17 +0000
@@ -35,11 +35,11 @@
 from bzrlib.decorators import needs_read_lock, needs_write_lock
 from bzrlib.repository import (
     InterRepository,
-    InterSameDataRepository,
     IsInWriteGroupError,
     RepositoryFormat,
     )
 from bzrlib.vf_repository import (
+    InterSameDataRepository,
     MetaDirVersionedFileRepository,
     MetaDirVersionedFileRepositoryFormat,
     VersionedFileCommitBuilder,

=== modified file 'bzrlib/repository.py'
--- bzrlib/repository.py	2011-05-08 11:20:31 +0000
+++ bzrlib/repository.py	2011-05-08 11:34:17 +0000
@@ -1378,6 +1378,8 @@
     supports_revision_signatures = True
     # Can the revision graph have incorrect parents?
     revision_graph_can_have_wrong_parents = None
+    # Does this format support rich root data?
+    rich_root_data = None
 
     def __repr__(self):
         return "%s()" % self.__class__.__name__
@@ -1851,29 +1853,6 @@
                 "different serializers")
 
 
-class InterSameDataRepository(InterRepository):
-    """Code for converting between repositories that represent the same data.
-
-    Data format and model must match for this to work.
-    """
-
-    @classmethod
-    def _get_repo_format_to_test(self):
-        """Repository format for testing with.
-
-        InterSameData can pull from subtree to subtree and from non-subtree to
-        non-subtree, so we test this with the richest repository format.
-        """
-        from bzrlib.repofmt import knitrepo
-        return knitrepo.RepositoryFormatKnit3()
-
-    @staticmethod
-    def is_compatible(source, target):
-        return InterRepository._same_model(source, target)
-
-
-InterRepository.register_optimiser(InterSameDataRepository)
-
 
 class CopyConverter(object):
     """A repository conversion tool which just performs a copy of the content.

=== modified file 'bzrlib/tests/test_repository.py'
--- bzrlib/tests/test_repository.py	2011-05-08 11:08:03 +0000
+++ bzrlib/tests/test_repository.py	2011-05-08 11:34:17 +0000
@@ -379,7 +379,11 @@
         # classes do not barf inappropriately when a surprising repository type
         # is handed to them.
         dummy_a = DummyRepository()
+        dummy_a._format = RepositoryFormat()
+        dummy_a._format.supports_full_versioned_files = True
         dummy_b = DummyRepository()
+        dummy_b._format = RepositoryFormat()
+        dummy_b._format.supports_full_versioned_files = True
         self.assertGetsDefaultInterRepository(dummy_a, dummy_b)
 
     def assertGetsDefaultInterRepository(self, repo_a, repo_b):
@@ -389,7 +393,7 @@
         no actual sane default in the presence of incompatible data models.
         """
         inter_repo = repository.InterRepository.get(repo_a, repo_b)
-        self.assertEqual(repository.InterSameDataRepository,
+        self.assertEqual(vf_repository.InterSameDataRepository,
                          inter_repo.__class__)
         self.assertEqual(repo_a, inter_repo.source)
         self.assertEqual(repo_b, inter_repo.target)
@@ -409,9 +413,11 @@
         dummy_a._serializer = repo._serializer
         dummy_a._format.supports_tree_reference = repo._format.supports_tree_reference
         dummy_a._format.rich_root_data = repo._format.rich_root_data
+        dummy_a._format.supports_full_versioned_files = repo._format.supports_full_versioned_files
         dummy_b._serializer = repo._serializer
         dummy_b._format.supports_tree_reference = repo._format.supports_tree_reference
         dummy_b._format.rich_root_data = repo._format.rich_root_data
+        dummy_b._format.supports_full_versioned_files = repo._format.supports_full_versioned_files
         repository.InterRepository.register_optimiser(InterDummy)
         try:
             # we should get the default for something InterDummy returns False

=== modified file 'bzrlib/vf_repository.py'
--- bzrlib/vf_repository.py	2011-05-08 11:20:31 +0000
+++ bzrlib/vf_repository.py	2011-05-08 11:34:17 +0000
@@ -2453,6 +2453,10 @@
 
     @staticmethod
     def is_compatible(source, target):
+        if not source._format.supports_full_versioned_files:
+            return False
+        if not target._format.supports_full_versioned_files:
+            return False
         # This is redundant with format.check_conversion_target(), however that
         # raises an exception, and we just want to say "False" as in we won't
         # support converting between these formats.
@@ -2475,10 +2479,6 @@
             return False
         if not target.bzrdir.transport.base.startswith('file:///'):
             return False
-        if not source._format.supports_full_versioned_files:
-            return False
-        if not target._format.supports_full_versioned_files:
-            return False
         return True
 
     def _get_trees(self, revision_ids, cache):
@@ -2787,7 +2787,32 @@
         return basis_id, basis_tree
 
 
+class InterSameDataRepository(InterRepository):
+    """Code for converting between repositories that represent the same data.
+
+    Data format and model must match for this to work.
+    """
+
+    @classmethod
+    def _get_repo_format_to_test(self):
+        """Repository format for testing with.
+
+        InterSameData can pull from subtree to subtree and from non-subtree to
+        non-subtree, so we test this with the richest repository format.
+        """
+        from bzrlib.repofmt import knitrepo
+        return knitrepo.RepositoryFormatKnit3()
+
+    @staticmethod
+    def is_compatible(source, target):
+        return (
+            InterRepository._same_model(source, target) and
+            source._format.supports_full_versioned_files and
+            target._format.supports_full_versioned_files)
+
+
 InterRepository.register_optimiser(InterDifferingSerializer)
+InterRepository.register_optimiser(InterSameDataRepository)
 
 
 def install_revisions(repository, iterable, num_revisions=None, pb=None):

